@@ -141,7 +141,6 @@
 #include "extensions.hpp"
 #include <algorithm>
 
-
 #define OPT_ARG_OFFSET 256
 #define OPT_WIDTH 100
 
@@ -163,16 +162,7 @@ class Option {
     typedef vec_string::iterator iter_opt;
     typedef vec_string::const_iterator citer_opt;
 
-    friend void             swap(Option &l, Option &r) {
-                             using std::swap;
-                             swap(l.id_, r.id_);
-                             swap(l.ot_, r.ot_);
-                             swap(l.val_, r.val_);
-                             swap(l.order_, r.order_);
-                             swap(l.name_, r.name_);
-                             swap(l.desc_, r.desc_);
-                             swap(l.go_, r.go_);
-                            }
+    friend SWAP(Option, id_, ot_, val_, order_, name_, desc_, go_)
 
  public:
     friend Getopt;
@@ -181,21 +171,35 @@ class Option {
     // standalone arguments; a bare '-' argument is set as a boolean option
     // options can be either boolean (w/o parameter) or parametric
     // standalone arguments are always parametric type
-    enum ArgKind { opt, arg };
-    enum OptType { boolean, parametric };
 
+    #define ARGKIND \
+             opt, \
+             arg
+    ENUMSTR(ArgKind, ARGKIND)
+
+    #define OPTTYPE \
+             boolean, \
+             parametric
+    ENUM(OptType, OPTTYPE)
+    #undef OPTTYPE
+
+    #define DMPOPT \
+             all, \
+             strike
+    ENUM(DumpOpt, DMPOPT)
+    #undef DMPOPT
 
                         Option(void) = delete;                  // DC deleted
-                        Option(short opt, Getopt *go):          // Custom constructor
+                        Option(int opt, Getopt *go):            // Custom constructor
                          id_{opt}, go_{go} { name_ = toupper(opt); }
                         Option(const Option &) = default;       // CC
-                        Option(Option && o)                     // MC
+                        Option(Option && o) noexcept            // MC
                          { swap(*this, o); }
-    Option &            operator=(Option o)                     // CA, MA
+    Option &            operator=(Option o) noexcept            // CA, MA
                          { swap(*this, o); return *this; }
 
     // User interface:
-    short int           id(void) const { return id_; }          // could be: 'a', 'x', 256, 257, ...
+    int                 id(void) const { return id_; }          // could be: 'a', 'x', 256, 257, ..
     ArgKind             kind(void) const
                          { return id_ < OPT_ARG_OFFSET? opt: arg; }
     OptType             type(void) const { return ot_; }
@@ -210,7 +214,7 @@ class Option {
     const std::string & description(void) const { return desc(); }
 
     // methods typically used after parsing
-    Option &            operator=(const std::string &s);        // append a new value & record a hit
+    Option &            operator=(const std::string &s);        // append a new value & record hit
     Option &            hit(void);                              // record a hit (& update order)
                         operator double(void) const
                          { return type() == boolean? hits(): atof(c_str()); }
@@ -218,7 +222,7 @@ class Option {
                         // size: the number of times the value was recorded for given parametric
                         // option/argument, size is always >= 1 (due to a reserved place for a
                         // default value)
-    size_t              hits(void) const { return order_.size()-1; }
+    size_t              hits(void) const { return order_.size() - 1; }
                         // hits: the number of times given option was given (boolean/parametric)
     size_t              ordinal(long i = -1) const
                          { return order_[mod_(order_.size(), i)]; }
@@ -232,20 +236,19 @@ class Option {
                         // str() and c_str() let inspect any recorded option parameters/arguments,
                         // including a default value (if set). by default, str() and c_str()
                         // return the last set value (including default)
-    Option &            reset(void)
-                         { val_.resize(1); order_.resize(1); return *this; }
-                        // reset to the state after setup and before parse
+    Option &            reset(void);
 
     // methods to iterate over parametric option values, skip default value
     iter_opt            begin(void) { return ++val_.begin(); }
     citer_opt           begin(void) const { return ++val_.cbegin(); }
     iter_opt            end(void) { return val_.end(); }
     citer_opt           end(void) const { return val_.cend(); }
+    Option &            dump(DumpOpt x=strike, std::ostream &xout = std::cerr);
 
  protected:
     const std::string & deflt_(void) const { return val_.front(); }
 
-    short int           id_{0};                                 // option symbol / arg's position
+    int                 id_{0};                                 // option symbol / arg's position
                         // for options, id_ is the ascii code of the options symbol,
                         // for arguments, id_ is based off 256, i.e arg[0] has id 256, etc
     OptType             ot_{boolean};                           // boolean or parametric (type)
@@ -264,6 +267,9 @@ class Option {
                         // for negative values -1, -2, ... returns size()-1, size()-2, etc
 };
 
+STRINGIFY(Option::ArgKind, ARGKIND)
+#undef ARGKIND
+
 
 
 Option & Option::bind(const std::string &deflt) {
@@ -275,6 +281,7 @@ Option & Option::bind(const std::string &deflt) {
 }
 
 
+
 Option & Option::name(const std::string &n) {
  // setup name of the argument and make option parametric
  name_ = n;
@@ -283,6 +290,7 @@ Option & Option::name(const std::string &n) {
 }
 
 
+
 Option & Option::operator=(const std::string &v) {
  // append a new value to the option (and make it parametric)
  val_.push_back(v);
@@ -292,6 +300,35 @@ Option & Option::operator=(const std::string &v) {
 }
 
 
+
+Option & Option::dump(DumpOpt x, std::ostream &xout) {
+ // dump all/hit options
+ std::string dlm{kind() == opt? " opt [": " arg ["};
+ dlm += kind() == opt? std::string{static_cast<char>(id())}: std::to_string(id() - OPT_ARG_OFFSET);
+ dlm += "]: ";
+
+ for(auto &v: val_) {
+  if(type() == boolean) {
+   if(x == strike and hits() == 0) return *this;                // skip printing non-hit booleans
+   xout << dlm << hits() << " hits" << std::endl;
+   return *this;
+  }
+  // parametric types here
+  if(&v == &val_[0]) {                                          // handle default value
+   if(x == strike or kind() == arg) continue;                   // skip printing default
+   xout << dlm << v << (v.empty()? "": " (default)");
+   dlm = v.empty()? "": ", ";
+   continue;
+  }
+  xout << dlm << v;
+  dlm = ", ";
+ }
+ if(dlm == ", " or dlm == "") xout << std::endl;
+ return *this;
+}
+
+
+
 std::ostream & operator<<(std::ostream & os, const Option & opt) {
  return opt.type() == Option::boolean? os << opt.hits(): os << opt.c_str();
 }
@@ -306,32 +343,28 @@ class Getopt {
  //   o after parsing: provides access to the recorded option values
  // parses provided options (argv),
  // allows auto generating help and usage outputs
-    typedef std::map<short, Option> map_opt;
+    typedef std::map<int, Option> map_opt;
     typedef map_opt::iterator iter_opt;
 
     class OptionsOrdered {
      // facilitates back-tracing every option to it's original position in user cli
-        friend void         swap(OptionsOrdered &l, OptionsOrdered &r) {
-                             using std::swap;
-                             swap(l.opt_, r.opt_);
-                             swap(l.cnt_, r.cnt_);
-                             swap(l.go_, r.go_);
-                            }
+        friend Getopt;
+        friend SWAP(OptionsOrdered, opt_, cnt_, go_)
 
      public:
-                            OptionsOrdered(void) = default;     // DC
-                            OptionsOrdered(short opt, size_t cnt, Getopt *go):  // Custom Const.
+                            OptionsOrdered(void) = default;                     // DC
+                            OptionsOrdered(int opt, size_t cnt, Getopt *go):    // for emplacement
                              opt_{opt}, cnt_{cnt}, go_(go) {}
                             OptionsOrdered(const OptionsOrdered &) = default;   // CC
-                            OptionsOrdered(OptionsOrdered && o) // MC
+                            OptionsOrdered(OptionsOrdered && o) noexcept        // MC
                              { swap(*this, o); }
-        OptionsOrdered &    operator=(OptionsOrdered o)         // CA, MA
+        OptionsOrdered &    operator=(OptionsOrdered o) noexcept                // CA, MA
                              { swap(*this, o); return *this; }
 
         size_t              count(void) const { return cnt_; }
         Option &            option(void) { return go_->om_.at(opt_); }
         const Option &      option(void) const { return go_->om_.at(opt_); }
-        short               id(void) const { return opt_; }
+        int                 id(void) const { return opt_; }
         Option::ArgKind     kind(void) const { return go_->om_.at(opt_).kind(); }
         Option::OptType     type(void) const { return go_->om_.at(opt_).type(); }
         const std::string & name(void) const { return go_->om_.at(opt_).name(); }
@@ -342,7 +375,7 @@ class Getopt {
         const std::string & str(void) const{ return option().str(count()); }
 
      private:
-        short               opt_;                               // option id: 'a', 'x', etc
+        int                 opt_;                               // option id: 'a', 'x', etc
         size_t              cnt_;                               // counter of an option (1,2,3,...)
         Getopt *            go_{nullptr};
     };
@@ -350,21 +383,10 @@ class Getopt {
     friend OptionsOrdered;
     friend Option;
 
-    friend void             swap(Getopt &l, Getopt &r) {
-                             using std::swap;
-                             swap(l.exception_, r.exception_);
-                             swap(l.throwException_, r.throwException_);
-                             swap(l.autohelp_, r.autohelp_);
-                             swap(l.variadic_, r.variadic_);
-                             swap(l.argc_, r.argc_);
-                             swap(l.argv_, r.argv_);
-                             swap(l.prolog_, r.prolog_);
-                             swap(l.epilog_, r.epilog_);
-                             swap(l.arguments_, r.arguments_);
-                             swap(l.prgname_, r.prgname_);
-                             swap(l.om_, r.om_);
-                             swap(l.ov_, r.ov_);
-                            }
+    friend COPY(Getopt, exception_, throw_exception_, autohelp_, variadic_,
+                        argc_, argv_, prolog_, epilog_, arguments_, prgname_, om_, ov_)
+    friend SWAP(Getopt, exception_, throw_exception_, autohelp_, variadic_,
+                        argc_, argv_, prolog_, epilog_, arguments_, prgname_, om_, ov_)
 
  public:
 
@@ -381,16 +403,27 @@ class Getopt {
     ENUMSTR(ThrowReason, THROWREASON)
 
 
-                        Getopt(void) = default;             // DC
-                        Getopt(const Getopt &) = default;   // CC
-                        Getopt(Getopt && go)                // MC
-                         { swap(*this, go); }
-    Getopt &            operator=(Getopt go)                // CA, MA
-                         { swap(*this, go); return *this; }
+                        Getopt(void) = default;                 // DC
+                        Getopt(const Getopt &go) {              // CC
+                         copy(*this, go);
+                         for(auto &o: *this) o.second.go_ = this;
+                         for(auto &o: ordinal()) o.go_ = this;
+                        }
+                        Getopt(Getopt && go) noexcept {         // MC
+                         swap(*this, go);
+                         for(auto &o: *this) o.second.go_ = this;
+                         for(auto &o: ordinal()) o.go_ = this;
+                        }
+    Getopt &            operator=(Getopt go) noexcept {         // CA, MA
+                         swap(*this, go);
+                         for(auto &o: *this) o.second.go_ = this;
+                         for(auto &o: ordinal()) o.go_ = this;
+                         return *this;
+                        }
 
 
     Getopt &            suppress_opterr(bool x=true) { opterr = !x; return *this; }
-    Getopt &            suppress_exceptions(void) { throwException_ = false; return *this; }
+    Getopt &            suppress_exceptions(void) { throw_exception_ = false; return *this; }
     Getopt &            suppress_autohelp(void) { autohelp_ = false; return *this; }
     Getopt &            variadic(bool x = true) { variadic_ = x; return *this; }
     void                parse(int argc, char *argv[], const char *fmt = nullptr);
@@ -405,18 +438,30 @@ class Getopt {
     Getopt &            help(void);                             // auto-generated help
     iter_opt            begin(void) { return om_.begin(); }
     iter_opt            end(void) { return om_.end(); }
-    Getopt &            reset(void)
-                         { for(auto &om: om_) om.second.reset(); ov_.clear(); return *this; }
+    Getopt &            reset(void) {
+                         for(auto &om: om_) om.second.reset();
+                         ov_.clear();
+                         #ifdef __linux__
+                          optind = 0;
+                         #endif
+                         return *this;
+                        }
     bool                defined(char opt) const { return om_.count(opt) == 1; }
     std::vector<OptionsOrdered> &
                         ordinal(void) { return ov_; }
     OptionsOrdered &    ordinal(size_t idx) { return ov_[idx]; }
+    Getopt &            dump(Option::DumpOpt x = Option::strike, std::ostream &xout = std::cerr) {
+                         xout << "dump of parsed options:" << std::endl;
+                         for(auto &kv: om_) kv.second.dump(x, xout);
+                         return *this;
+                        }
+
 
     EXCEPTIONS(ThrowReason)
 
  protected:
     char                exception_;                             // option that generated exception
-    bool                throwException_{true};                  // let throw is a default behavior
+    bool                throw_exception_{true};                 // let throw is a default behavior
     bool                autohelp_{true};
     bool                variadic_{true};
                         // last argument can be variadic (accept multiple values), however,
@@ -456,6 +501,22 @@ Option & Option::hit(void) {
  // and update order of the option itself
  go_->ov_.emplace_back(id(), hits() + 1, go_);                  // update order of the option
  order_.push_back(go_->ov_.size() - 1);                         // update my order reference
+ if(kind() == arg) ++go_->arguments_;                           // if it's argument, increase cnt
+ return *this;
+}
+
+
+
+Option & Option::reset(void) {
+ // reset to the state after setup and before parse
+ val_.resize(1);
+ order_.resize(1);
+
+ decltype(go_->ov_) new_ov;
+ for(auto &opt: go_->ov_)
+  if(opt.id() != id()) new_ov.push_back(std::move(opt));
+ go_->ov_ = std::move(new_ov);
+
  return *this;
 }
 
@@ -464,20 +525,19 @@ Option & Option::hit(void) {
 // for description of variables optreset, optind, optarg and optopt refer to description
 // of getopt() function. Those are external parameters which getopt sets and is based on
 
-//void Getopt::update_order_(short option) {
-// record given option into order vector - to keep track or options original order
-//}
-
-
 void Getopt::parse(int argc, char *argv[], const char *f) {
  // 1. recover format string from defined options (if any) - so that getopt could parse it
  // 2. generate auto-help (if -h given) and setup program name
  // 3. process input 'argv' array (via getopt call), then post process standalone arguments
- #ifndef __linux__
- optreset = 1;                                                  // allow multiple parsing
- #endif
- optind = 1;                                                    // - linux way to use optreset
  std::string fmt = recoverFormat_(f);                           // recovered format
+ #ifdef __linux__
+  if(optind != 0) optind = 1;                                   // linux way to rescan arguments
+  if(fmt.front() != '+') fmt = "+" + fmt;
+  // '+' in optstring: description at http://man7.org/linux/man-pages/man3/getopt.3.html
+ #else
+  optreset = 1;                                                 // allow multiple parsing
+  optind = 1;
+ #endif
 
  if(autohelp_) {
   if(fmt.find('h') != std::string::npos)
@@ -493,9 +553,13 @@ void Getopt::parse(int argc, char *argv[], const char *f) {
 
  parseInputArgs_(argc, argv, fmt);                              // process all inputs in argc/argv
  processStandalone_(argc, argv);                                // now process all args
+ #ifdef __linux__
+  optind = 0;
+ #endif
 }
 
 
+
 void Getopt::parseInputArgs_(int argc, char *argv[], const std::string &fmt) {
  // parse all provided arguments with getopt():
  // extract option one by one, create Option container out of extracted values
@@ -506,10 +570,10 @@ void Getopt::parseInputArgs_(int argc, char *argv[], const std::string &fmt) {
    throw EXP(opt_argument_missing);                             // missing argument, forced ':'
 
   if(option == '?')                                             // '?' means invalid option
-   if(throwException_) {                                         // if throwing is allowed
-    exception_ = optopt;                                        //   optopt = either unknown option,
-    throw fmt.find(exception_) == std::string::npos?            //   (if not found in fmt string)
-           EXP(invalid_option): EXP(opt_argument_missing);      //   or missing arg, throw anyway
+   if(throw_exception_) {                                       // if throwing is allowed
+    exception_ = optopt;                                        //  optopt = either unknown option
+    throw fmt.find(exception_) == std::string::npos?            //  (if not found in fmt string)
+           EXP(invalid_option): EXP(opt_argument_missing);      //  or missing arg, throw anyway
    }                                                            // otherwise let user handle it
 
   if(optarg)                                                    // if option has argument
@@ -526,6 +590,7 @@ void Getopt::parseInputArgs_(int argc, char *argv[], const std::string &fmt) {
 }
 
 
+
 void Getopt::processStandalone_(int argc, char *argv[]) {
  // process only standalone arguments and bare '-' input redirector
  // last variadic argument may collect multiple values
@@ -533,7 +598,7 @@ void Getopt::processStandalone_(int argc, char *argv[]) {
  int ha = 0;                                                    // ha: hyphen adjust
 
  for(int i = optind; i<argc; i++) {                             // process all standalone arguments
-  short idx = OPT_ARG_OFFSET + i - optind - ha;                 // arg index in the map (adjusted)
+  int idx = OPT_ARG_OFFSET + i - optind - ha;                   // arg index in the map (adjusted)
   if(argv[i][0] == '-' and argv[i][1] == '\0') {                // if arg is a bare qualifier '-'
    om_.at('-').hit();
    ++ha;
@@ -545,15 +610,14 @@ void Getopt::processStandalone_(int argc, char *argv[]) {
     throw EXP(too_many_arguments);                              // throw if not variadic
    if(idx == OPT_ARG_OFFSET)                                    // if it's a first argument
     om_.emplace(OPT_ARG_OFFSET, Option{OPT_ARG_OFFSET, this});  // record it,
-   om_.rbegin()->second = argv[i];                              // select last index (it's variadic)
+   om_.rbegin()->second = argv[i];                              // select last index, it's variadic
   }
   else                                                          // argument was defined
    om_.at(idx) = argv[i];
-  ++arguments_;
  } // for(...
 
  // now check if any mandatory arguments are missing
- for(auto &o: om_) {                                             // go through all the parsed option
+ for(auto &o: om_) {                                            // go through all the parsed option
   auto & opt = o.second;
   if(opt.kind() == Option::arg)                                 // argument defined/described
    if(opt.size() == 1 and opt.str(0).empty())                   // but its value is unset
@@ -562,6 +626,7 @@ void Getopt::processStandalone_(int argc, char *argv[]) {
 }
 
 
+
 class Option & Getopt::operator[](char opt) {
  // user access to the option. If undefined option is accessed after parsing
  // the program will throw (deemed to be a programming error)
@@ -580,7 +645,7 @@ class Option & Getopt::operator[](char opt) {
 class Option & Getopt::operator[](int opt) {
  // user access to the standalone arguments
 
- short idx = OPT_ARG_OFFSET + opt;
+ int idx = OPT_ARG_OFFSET + opt;
 
  if(om_.find(idx) != om_.end())                                 // argument is found (defined)
   return om_.at(idx);
@@ -595,6 +660,7 @@ class Option & Getopt::operator[](int opt) {
 }
 
 
+
 Getopt & Getopt::usage(void) {
  // print usage on the console: first go options then arguments
  std::stringstream ss;
@@ -613,11 +679,12 @@ Getopt & Getopt::usage(void) {
 }
 
 
+
 void Getopt::usagePrintOptions_(std::stringstream &ss, int indent) {
  // print options only here
  std::stringstream os;
  os << " [-";
- for(auto &m: om_) {                                             // print boolean options first
+ for(auto &m: om_) {                                            // print boolean options first
   auto &opt = m.second;
   if(opt.type() != Option::boolean) continue;                   // process options w/a params here
   if(opt.id() == '-' ) continue;                                // skip input redirector
@@ -627,7 +694,7 @@ void Getopt::usagePrintOptions_(std::stringstream &ss, int indent) {
  outputOpt_(ss, indent, os.str());
  os.str(std::string());
 
- for(auto &m: om_) {                                             // print options with parameters
+ for(auto &m: om_) {                                            // print options with parameters
   auto &opt = m.second;
   if(opt.kind() == Option::arg) continue;
   if(opt.type() != Option::parametric) continue;
@@ -638,6 +705,7 @@ void Getopt::usagePrintOptions_(std::stringstream &ss, int indent) {
 }
 
 
+
 void Getopt::usagePrintArgs_(std::stringstream &ss, int indent) {
  // print arguments taking into account possible variadic behavior of the last argument
  bool noneDefined = true;
@@ -665,21 +733,22 @@ void Getopt::usagePrintArgs_(std::stringstream &ss, int indent) {
 }
 
 
+
 Getopt & Getopt::help(void) {
  // print help screen
  std::cout << prolog_ << std::endl;
 
- if(om_.size() - arguments()) {                                  // if any options deinfed
+ if(om_.size() - arguments()) {                                 // if any options deinfed
   std::cout << "optional arguments:" << std::endl;
 
-  for(auto &m: om_) {                                            // first print boolean options
+  for(auto &m: om_) {                                           // first print boolean options
    auto & opt = m.second;
    if(opt.id() == '-') continue;
    if(opt.type() == Option::boolean)
     optionHelpLine_(opt);
   }
 
-  for(auto &m: om_) {                                            // print parametrical options
+  for(auto &m: om_) {                                           // print parametrical options
    auto & opt = m.second;
    if(opt.kind() == Option::opt and opt.type() == Option::parametric)
     optionHelpLine_(opt);
@@ -701,6 +770,7 @@ Getopt & Getopt::help(void) {
 }
 
 
+
 void Getopt::optionHelpLine_(const Option & opt) {
  // printing options help line
  std::stringstream out;
@@ -724,6 +794,7 @@ void Getopt::optionHelpLine_(const Option & opt) {
 }
 
 
+
 std::string Getopt::recoverFormat_(const char *f) {
 // recovers getopt's format string from given binds/names/descriptions (in om_)
 // if both fmt and option (om_) defined, merge those
@@ -732,28 +803,28 @@ std::string Getopt::recoverFormat_(const char *f) {
  if(om_.empty() and fmt.empty())
   throw EXP(format_missing);                                    // either must be defined
 
- for(auto &m: om_) {                                             // recover fmt from defined options
+ for(auto &m: om_) {                                            // recover fmt from defined options
   auto & opt = m.second;
   if(opt.kind() == Option::arg)
    continue;                                                    // process only options for now
 
-  if(fmt.find(opt.id()) == std::string::npos) {                  // if option is not in fmt yet - add
+  if(fmt.find(opt.id()) == std::string::npos) {                 // if option is not in fmt yet: add
    fmt += opt.id();
    if(opt.type() == Option::parametric)
     fmt += ':';
    continue;
   }
 
-  auto ops = fmt.find(opt.id());                                // option defined, check consistency
+  auto ops = fmt.find(opt.id());                                // option exists, check consistency
   if(not ((opt.type() == Option::boolean) xor (fmt[ops+1] == ':')))
    throw EXP(inconsistent_format_string);
  } // for(...
 
- auto itr = om_.begin();                                       // point to last found option
- for(auto ops: fmt)                                            // ensure all ops are booked, per fmt
+ auto itr = om_.begin();                                        // point to last found option
+ for(auto ops: fmt)                                             // ensure ops are booked, per fmt
   if(ops == ':')
-   itr->second.bind();                                         // make parametric
-  else                                                         // ops == ':', option is parametric
+   itr->second.bind();                                          // make parametric
+  else                                                          // ops == ':', option is parametric
    if(om_.count(ops) == 0)
     itr = om_.emplace(ops, Option{ops, this}).first;
    else
@@ -763,6 +834,7 @@ std::string Getopt::recoverFormat_(const char *f) {
 }
 
 
+
 unsigned Getopt::countArgs_(void) {
  // counts defined standalone args (i.e. before parsing)
  return
@@ -771,6 +843,7 @@ unsigned Getopt::countArgs_(void) {
 }
 
 
+
 void Getopt::outputOpt_(std::stringstream &ss, int indent, const std::string &str) {
  // in usage: output minding max width (OPT_WIDTH)
  if(static_cast<size_t>(ss.tellp()) + str.size() <= OPT_WIDTH)
